# Copyright 2010 A4 Developers.  This software is licensed under the
# GNU General Public License version 3 (see the file COPYING).

import cairo
import gtk
import math
from numpy import array, dot
from numpy.linalg import inv
from a4lib.player import GtkCairoPlayer
from a4lib.region import Region

from presentation_objects import RectObject, TextObject, PolaroidObject


def object_class_from_name(name):
    obj_dict = { 'rect': RectObject, 'text': TextObject,
                 'polaroid': PolaroidObject }
    return obj_dict[name]


class Editor(GtkCairoPlayer):

    def __init__(self, file_name, drawing_area, treeview, force_loading=False):
        GtkCairoPlayer.__init__(self, file_name, drawing_area, force_loading)
        self.treeview = treeview
        self.model = gtk.ListStore(int, str, gtk.gdk.Pixbuf)

        self.treeview.set_model(self.model)
        self.treeview.set_text_column(1)
        self.treeview.set_pixbuf_column(2)

        self.treeview.set_columns(1)
        self.treeview.set_item_width(120)
        self.treeview.set_size_request(130, -1)

        ## used to keep status of mouse actions:
        ## value currently used: [None,'start','moving']
        self.action_status = None
        ## tells if we should draw the rect of the path.
        self.show_path = None
        ## tell the tool we're using and how we should behave.
        self.editor_modes = ['select', 'path', 'objects']
        self.editor_mode = 'select'
        self.new_object_name = None
        ## the selected object id:
        self.current_object = None

        for i in xrange(0, len(self.presentation.path)):
            region = self.presentation.get_path_region(i)
            self.draw_region_on_iconview(region, i)

    def set_show_path(self, flag=True):
        self.show_path = flag

    # objects mode: rect/polaroid
    def set_next_new_object(self, obj_name):
        self.new_object_name = obj_name

    # select/objects
    def set_editor_mode(self, status='select'):
        if status in self.editor_modes:
            self.editor_mode = status

    def on_area_button_press_event(self, widget, event):
        #print "Finestra", self.area.window.get_size()
        #print "Current", self.current_region
        #print "Svg", self.presentation.width,self.presentation.height
        event_point = (event.x, event.y)
        winsize = self.area.window.get_size()
        cr = self.current_region
        point = cr.get_point_from_window_coordinates(winsize,event_point)
        #print "Event Point",event_point,
        #print " while Point",point

        self.selection_start_position = (event.x, event.y)
        self.action_status = 'start'
        if self.editor_mode == 'select':
            pass  # presentation.select(something)
        elif self.editor_mode == 'objects':
            cls = object_class_from_name(self.new_object_name)
            print cls
            self.current_object = cls( x=point[0], y=point[1],
                                      rotate=self.current_region.rotate)
            self.presentation.add_object(self.current_object)
        else:
            GtkCairoPlayer.on_area_button_press_event(self, widget, event)

    def on_area_button_release_event(self, widget, event):
        GtkCairoPlayer.on_area_button_release_event(self, widget, event)
        if self.editor_mode == 'path' and event.state & gtk.gdk.CONTROL_MASK:
            rect = (self.selection_start_position, self.mouse_position)
            self.add_treeview_icon(rect)
        elif self.editor_mode == 'objects':
            pass
        self.action_status = None


    def on_area_motion_notify_event(self, widget, event):
        if self.action_status is None:
            GtkCairoPlayer.on_area_motion_notify_event(self, widget, event)
        else:
            if self.editor_mode == 'path':
                if event.state & gtk.gdk.CONTROL_MASK:
                    self.mouse_position = (event.x, event.y)
                    self.area.queue_draw()
            elif self.editor_mode == 'objects' and self.action_status:
                if self.current_object:
                    winsize = self.area.window.get_size()
                    cr = self.current_region
                    scale = self.current_region.get_window_scale_factor(winsize)
                    w = (event.x - self.selection_start_position[0]) / scale
                    h = (event.y - self.selection_start_position[1]) / scale
                    self.current_object.set_size(w,h)
                self.mouse_position = (event.x, event.y)
                self.area.queue_draw()
            self.action_status = 'moving'

    def on_area_key_press(self, widget, event):
        if self.current_object:
            if True:  # premo tutto tranne esc:
                key = gtk.gdk.keyval_name(event.keyval)
                self.current_object.character_pressed(key)
                self.area.queue_draw()
            elif False:  # premo esc
                self.action_status = None
            return True

    def draw_selection_box(self, context, region=None):
        context.save()
        if not region:
            context.set_matrix(cairo.Matrix(1, 0, 0, 1, 0, 0))
            x0 = self.selection_start_position[0]
            y0 = self.selection_start_position[1]
            x1 = self.mouse_position[0]
            y1 = self.mouse_position[1]
        else:
            context.translate(region.xc, region.yc)
            context.rotate(region.rotate)
            context.translate(-region.xc, -region.yc)
            x0 = region.xc - region.width / 2
            y0 = region.yc - region.height / 2
            x1 = region.xc + region.width / 2
            y1 = region.yc + region.height / 2

        context.set_source_rgb(0, 0xff, 0)
        context.move_to(x0, y0)
        context.line_to(x1, y0)
        context.line_to(x1, y1)
        context.line_to(x0, y1)
        context.line_to(x0, y0)
        context.set_line_width(0.2)
        context.stroke()

        context.rectangle(x0, y0, x1 - x0, y1 - y0)
        context.set_source_rgba(0, 0, 0.5, 0.50)
        context.fill()
        context.restore()

    def render(self, context):
        #   render a transparent box (should indicate the selection)
        # render already existant path points.
        # render an icon for rotating the whole image.
        GtkCairoPlayer.render(self, context)
        if self.show_path:
            for i in xrange(0, len(self.presentation.path)):
                region = self.presentation.get_path_region(i)
                self.draw_selection_box(context, region)
            if self.editor_mode == 'path' and self.action_status:
                self.draw_selection_box(context)
        #self.rotate_icon.render(context, self.area.window.get_size() )

    def add_treeview_icon(self, rect=None):
        ## * trasforma rect in Regione
        win_size = self.area.window.get_size()
        scale = self.current_region.get_window_scale_factor(win_size)
        translate = self.current_region.get_window_fit_vector(win_size)
        translate = translate[0] / 2.0, translate[1] / 2.0
        rect2 = (((rect[0][0] - translate[0]) / scale,
                  (rect[0][1] - translate[1]) / scale),
                 ((rect[1][0] - translate[0]) / scale,
                  (rect[1][1] - translate[1]) / scale))
        xc = (rect2[0][0] + rect2[1][0]) / 2 - self.current_region.width / 2
        yc = (rect2[0][1] + rect2[1][1]) / 2 - self.current_region.height / 2
        r = self.current_region.rotate
        xc1 = math.cos(r) * xc - math.sin(r) * yc
        yc1 = math.sin(r) * xc + math.cos(r) * yc
        region = Region(
                    self.current_region.xc + xc1,
                    self.current_region.yc + yc1,
                    (rect2[1][0] - rect2[0][0]),
                    (rect2[1][1] - rect2[0][1]),
                    self.current_region.rotate)
        #print "Finestra", self.area.window.get_size()
        #print "Current", self.current_region
        #print "Aggiungo", region
        #print
        ## * metti la regione in lista
        self.presentation.path.append(region)
        ## * usa la regione per la pixbuf.
        self.draw_region_on_iconview(region, len(self.presentation.path) - 1)

    def draw_region_on_iconview(self, region, position=0):
        pixbuf_height = abs(int(100 * region.height / region.width))
        pixbuf = gtk.gdk.Pixbuf(gtk.gdk.COLORSPACE_RGB, True, 8, 100,
                    pixbuf_height)
        pix_data = pixbuf.get_pixels_array()
        pixbuf_size = (pixbuf.get_width(), pixbuf.get_height())
        surface = cairo.ImageSurface.create_for_data(pix_data, cairo.FORMAT_ARGB32,
                     pixbuf_size[0], pixbuf_size[1], pixbuf.get_rowstride())
        cr = cairo.Context(surface)
        region.render(cr, self.presentation, pixbuf_size)

        self.model.append([position, str(position), pixbuf])
